<?php
/**
 * Defines all the action hooks that we execute
 *
 *
 * @since             1.0
 * @package           Ajax Live Search
 *
 */
 	/**
	 * Callback to the get_search_form filter, 
	 * allows us to edit search forms generated using get_search_form()
	 *
	 * @since 1.0
	 *
	 * @param $form string The generated markup for the search form
	 *
	 * @return string Markup for the search form
	 */
	function als_form_filter( $form ) {	

		// Disable browser autocompletes
		$form = str_replace( 'name="s"', 'name="s" autocomplete="off" ', $form );		
		return $form;
				
	}
	add_filter( 'get_search_form', 'als_form_filter' );

	
	/**
	 * Filter to add extra search conditions
	 *
	 * @since 1.0
	 *
	 * @param $conditions array An array of existing conditions
	 *
	 * @return array an array of the modified conditions
	 */

	function als_search_conditions($conditions) {
		$conditions['posts'] .=' AND post_type NOT IN ("nav_menu_item", "attachment")';
		
		return $conditions;
	}
	add_filter( 'als_search_conditions', 'als_search_conditions' );


	/**
	 * Filter to allow or deny the use of our inbuilt search function
	 *
	 * @since 1.0
	 *
	 * @param $allow bool
	 *
	 * @return bool
	 */

	function als_use_custom_search($allow) {
			if (!isset($_GET['s'])) {
				$allow = false;						// Only works on search pages
			}
			
			return $allow;
	}
	add_filter( 'als_use_custom_search', 'als_use_custom_search' );

	
	/**
	 * Filters indexable post types
	 *
	 * @param $types an array of current post types
	 *
	 * @since 1.0
	 * @return array
	 */

	function als_index_post_types($types = array()) {
		$post_types = get_option( 'als_post_types', als_post_types(false) ); //set true for array
		if(empty($post_types)){
			return $types;
		}
		
		return explode(' ', als_remove_punct($post_types));
	}
	add_filter( 'als_index_post_types', 'als_index_post_types' );
	
	/**
	 * Filters search query to excecute the author_in modifier
	 *
	 * @param $array array of author ids currently being searched
	 * @param $q search term
	 *
	 * @since 1.1.0
	 * @return array
	 */

	function als_author_in($array, $q) {
		$authors = als_query_modifier( 'author_in:', $q );
		
		if(!$authors ){
			
			return $array;
			
		}
		return (array_merge( $array, explode(',', $authors)));
	}
	add_filter( 'als_author_in', 'als_author_in', 10, 2 );
	
	/**
	 * Filters search query to excecute the author_not_in modifier
	 *
	 * @param $array array of author ids currently being excluded
	 * @param $q search term
	 *
	 * @since 1.1.0
	 * @return array
	 */

	function als_author_not_in($array, $q) {
		$authors = als_query_modifier( 'author_not_in:', $q );
		
		if(!$authors ){
			
			return $array;
			
		}
		return (array_merge( $array, explode(',', $authors)));
	}
	add_filter( 'als_author_not_in', 'als_author_not_in', 10, 2 );
	
	/**
	 * Filters search query to excecute the author_name modifier
	 *
	 * @param $string name of author being searched for
	 * @param $q search term
	 *
	 * @since 1.1.0
	 * @return string
	 */

	function als_author_name($string, $q) {
		$author = als_query_modifier( 'author:', $q );
		
		if(!$author ){
			
			$author = als_query_modifier( '@', $q );
			
		}
		
		if(!$author ){
			
			return $string;
			
		}
		return $author;
	}
	add_filter( 'als_author_name', 'als_author_name', 10, 2 );
	
	/**
	 * Filters search query to excecute the cat modifier
	 *
	 * @param $int id of category being searched on
	 * @param $q search term
	 *
	 * @since 1.1.0
	 * @return int
	 */

	function als_cat($int, $q) {
		$category = als_query_modifier( 'cat:', $q );
		
		if(!$category ){
			
			return $int;
			
		}
		return $category;
	}
	
	/**
	 * Filters search query to excecute the category modifier
	 *
	 * @param $string name of category being searched on
	 * @param $q search term
	 *
	 * @since 1.1.0
	 * @return string
	 */

	function als_category_name($string, $q) {
		$category = als_query_modifier( 'category:', $q );
		
		if(!$category ){
			
			return $string;
			
		}
		return category;
	}
	add_filter( 'als_cat', 'als_cat', 10, 2 );
	add_filter( 'als_category_name', 'als_category_name', 10, 2 );
	
	/**
	 * Filters search query to excecute the tagged modifier
	 *
	 * @param $string name of tag being searched on
	 * @param $q search term
	 *
	 * @since 1.1.0
	 * @return string
	 */

	function als_tag($string, $q) {
		$category = als_query_modifier( 'tagged:', $q );
		
		if(!$category ){
			
			return $string;
			
		}
		return category;
	}
	add_filter( 'als_tag', 'als_tag', 10, 2 );
	
	/**
	 * Filters search query to excecute the category_in modifier
	 *
	 * @param $array array of category ids currently being searched for
	 * @param $q search term
	 *
	 * @since 1.1.0
	 * @return array
	 */

	function als_category_in($array, $q) {
		$cats = als_query_modifier( 'category_in:', $q );
		
		if(!$cats ){
			
			return $array;
			
		}
		return (array_merge( $array, explode(',', $cats)));
	}
	add_filter( 'als_category_in', 'als_category_in', 10, 2 );
	
	/**
	 * Filters search query to excecute the category_not_in modifier
	 *
	 * @param $array array of category ids currently being excluded
	 * @param $q search term
	 *
	 * @since 1.1.0
	 * @return array
	 */

	function als_category_not_in($array, $q) {
		$cats = als_query_modifier( 'category_not_in:', $q );
		
		if(!$cats ){
			
			return $array;
			
		}
		return (array_merge( $array, explode(',', $cats)));
	}
	add_filter( 'als_category_not_in', 'als_category_not_in', 10, 2 );
	
	
	/**
	 * Filters search query to excecute the before: and after: modifier
	 *
	 * @param $array array of date params currently being used
	 * @param $q search term
	 *
	 * @since 1.1.0
	 * @return array
	 */

	function als_date_query($array, $q) {
		$before = als_query_modifier( 'before:', $q );
		$after = als_query_modifier( 'after:', $q );
		$date = array();
		
		if($before ){
			
			$date['before'] = $before; 
			
		}
		
		if($after ){
			
			$date['after'] = $after; 
			
		}
		$full = array($date);
		return (array_merge( $array, $full));
	}
	add_filter( 'als_date_query', 'als_date_query', 10, 2 );

	/**
	 * Filters search query to excecute the post_types modifier
	 *
	 * @param $array array of searchable post_types
	 * @param $q search term
	 *
	 * @since 1.1.0
	 * @return array
	 */

	function als_wp_query_post_types($array, $q) {
		$types = als_query_modifier( 'post_types:', $q );
		$array = array_unique($array); 
		if(!$types ){		
			return $array;
		
		} 

		$types = explode(',', $types);
		$to_show = array();
		
		for($i=0; $i<count($types); $i++){
			
			if(in_array($types[$i], $array)){
				
				$to_show[] = $types[$i];
				
			}
			
		}

		return $to_show;
	}
	add_filter( 'als_wp_query_post_types', 'als_wp_query_post_types', 10, 2 );
	
	
	/**
	 * Adds more setting fields to the searching tab
	 *
	 * @param $int integer of posts per page
	 * @param $q search term
	 *
	 * @since 1.1.0
	 * @return array
	 */

	function als_searching_settings($fields) {

		$last 		= array_pop($fields);
		$post_types = array_unique(als_index_post_types());	
		
		$extra_fields=array();
		for($i=0; $i<count($post_types); $i++){
			if(strlen($post_types[$i]) != 0) {
			$extra_fields[] = array(
				'title'    => $post_types[$i] . ' weight',
				'id'       => "als_{$post_types[$i]}_weight",
				'css'      => 'width: 48px;',
				'default'  => '1',
				'type'     => 'text',
				'desc_tip' =>  true,
			);
			}
			
		}
		
		$extra_fields[] = $last;
		return array_merge($fields, $extra_fields);
	}
	add_filter( 'als_searching_settings', 'als_searching_settings');

/**
 * Adds custom classes to the array of body classes.
 *
 * @since 2.1.0
 *
 * @param array $classes Classes for the body element.
 * @return array (Maybe) filtered body classes.
 */
function als_body_classes( $classes ) {
	$classes[] = 'ajax-live-search als-lite';
	

	return $classes;
}
add_filter( 'body_class', 'als_body_classes' );